/* port.h - include file for portability additions */

#ifndef _PORT_H_
#define _PORT_H_

#include <sys/types.h>

size_t my_strlcpy( char* dst, const char* src, size_t size );
size_t my_strlcat( char* dst, const char* src, size_t size );

#endif /* _PORT_H_ */
