/* version.h - version defines for e */

#ifndef _VERSION_H_
#define _VERSION_H_

#define E_PROGRAM "e"
#define E_VERSION "1.05 02may2025"
#define E_URL "http://www.acme.com/software/e/"

#endif /* _VERSION_H_ */
